#
# lico-fishing.R,  7 Aug 22
#
# Data from:
# https://github.com/pigna90/linuxcounter_stats

library("colorspace")
library("plyr")
# library("semver")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(3)


# Does not consider the possibility that a subset are the same
is_same_computer=function(df)
{
if (nrow(df) == 1)
   return(FALSE)

same_vals=function(cols) return (length(unique(cols)) == 1)

return (same_vals(df$numCores) &
	same_vals(df$class) & same_vals(df$cpu) &
	same_vals(df$country) & same_vals(df$distribution))
}


# Fit exponential model to memory over time
fit_mem_mod=function(df, col_str)
{
lm_mod=glm(log(memory) ~ days, data=df, subset=(memory > 0))
print(summary(lm_mod))
pred=predict(lm_mod, newdata=data.frame(days=xrange))
lines(start_date+xrange, exp(pred), col=col_str)
}


# Return percentage of swap/memory ratio values
mem_swap_ratio=function(df)
{
df=subset(df, (memory > 0) & (swap > 0))
t=count(round(df$swap/df$memory, digits=1))
q=t[order(t$freq, decreasing=TRUE), ]
q$perc=100*q$freq/nrow(df)

return(q)
}


lico=read.csv("lico.csv", as.is=TRUE, sep=";")
lver=read.csv("linux-kernel-ver.csv", as.is=TRUE)
lver$date=as.Date(lver$date, format="%d-%b-%Y")

t=merge(lico, lver, by.x="kernel", by.y="version", all.x=TRUE)

start_date=min(t$date, na.rm=TRUE)
t$days=as.numeric(t$date-start_date)+1

# q=subset(t, is.na(date)) # There are some not-listed versions
# nrow(q)
# unique(q$kernel)
 
lico=t

scId=ddply(lico, .(userId), is_same_computer)
# length(which(scId$V1))
sct=subset(scId, V1)
sc=subset(lico, userId %in% sct$userId)

# Rows where every user has two entries

t=count(lico$userId)
# q=count(t$freq)
# q
#     x freq
# 1   1 3348
# 2   2  922
# 3   3  378
# 4   4  183
# 5   5   95
lt=subset(t, freq == 2)$x

lico2=subset(lico, userId %in% lt)

scId2=ddply(lico2, .(userId), is_same_computer)
sc2t=subset(scId2, V1)
sc2=subset(lico2, userId %in% sc2t$userId) # small enough to ignore

plot(sc2$memory, sc2$swap, log="xy", col=pal_col[2],
        xlab="Memory", ylab="Swap")


table(lico$class)

nv_lico=subset(lico, (class != "virtualmachine") & (class != "embedded"))

lap=subset(lico, class == "laptop")
ser=subset(lico, class == "server")
work=subset(lico, class == "workstation")

lapserwork=subset(lico, (class == "laptop") | (class == "server") | (class == "workstation"))

xrange=1:max(lico$days, na.rm=TRUE)

plot(ser$date, ser$memory, log="y", col=pal_col[1],
	xlab="Date", ylab="Memory (Mb)")
points(lap$date, lap$memory, col=pal_col[2])
points(work$date, work$memory, col=pal_col[3])

sm_mod=fit_mem_mod(ser, pal_col[1])
lm_mod=fit_mem_mod(lap, pal_col[2])
wm_mod=fit_mem_mod(work, pal_col[3])

legend("topleft", legend=c("Server", "Laptop", "Workstation"),
                        fill=pal_col, border="white", bty="n")


plot(ser$date, ser$numCores, log="y", col=pal_col[1],
	xlab="Date", ylab="Cores")
points(lap$date, lap$numCores, col=pal_col[2])
points(work$date, work$numCores, col=pal_col[3])

cr_mod=glm(numCores ~ days+I(days^2), data=lico)
summary(cr_mod)


plot(lapserwork$memory, lapserwork$swap, log="xy", col=pal_col[2],
	xlab="Memory", ylab="Swap")

lines(c(1, 1e9), c(1, 1e9), col=pal_col[3])

li_mod=glm(log(swap) ~ log(memory), data=lapserwork, subset=(memory > 0) & (swap > 0))
summary(li_mod)

xbounds=exp(seq(0, log(1e9), by=0.2))
pred=predict(li_mod, newdata=data.frame(memory=xbounds))

lines(xbounds, exp(pred), col=pal_col[1])

legend("bottomright", legend=c("Fitted regression line", "Data", "Swap == Memory"),
                        fill=pal_col, border="white", bty="n")


# Swap space size is memory
# times 2 for memory less 2G
# times 1 for mem 2-8G
# times 0.5 for mem greater 8G

sw_mod=glm(log(swap) ~ log(memory)+log(diskSpace)+numCores, data=lapserwork,
						subset=(memory > 0) & (swap > 0) & (diskSpace > 0))
summary(sw_mod)


sm_ratio=mem_swap_ratio(ser)
head(sm_ratio)
lm_ratio=mem_swap_ratio(lap)
head(lm_ratio)
wm_ratio=mem_swap_ratio(work)
head(wm_ratio)

